class ActionStatus:
    status = ""
    errorCode = ""
    message = ""

    def __init__(self, status="", errorCode="", message=""):
        self.status = status
        self.errorCode = errorCode
        self.message = message



from pyopencell.responses.action_status import ActionStatus


class Response:
    action_status = {}

    def __init__(self, cls, **kwargs):
        self.action_status = ActionStatus(**kwargs.get("actionStatus"))

        instance = cls(**kwargs[cls._name])
        setattr(self, cls._name, instance)



from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.response import Response
from pyopencell.responses.action_status import ActionStatus


class Access(BaseResource):
    _name = "access"
    _url_path = "/account/access"

    code = ""
    subscription = ""
    startDate = ""
    endDate = ""
    customFields = []
    disabled = False

    @classmethod
    def get(cls, accessCode=None, subscriptionCode=None):
        """
        Returns an access instance obtained by code or by subscription code.

        :param accessCode:
        :param subscriptionCode:
        :return: Access:
        """
        # Only one is required. If the two arguments are missing, a error was returned by the API
        query_params = {}
        if accessCode:
            query_params['accessCode'] = accessCode
        if subscriptionCode:
            query_params['subscriptionCode'] = subscriptionCode
        response_data = Client().get(cls._url_path, **query_params)

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return Response(cls, **response_data)

    @classmethod
    def create(cls, **kwargs):
        """
        Creates an access instance.

        :param kwargs:
        :return:
        """
        response_data = Client().post(cls._url_path, kwargs)

        return ActionStatus(**response_data)



class Paging:
    fullTextFilter = ""
    filters = {}
    fields = ""
    offset = 0
    limit = 0
    sortBy = "id"
    sortOrder = "ASCENDING"  # "ASCENDING" or "DESCENDING"
    totalNumberOfRecords = 0
    loadReferenceDepth = 0

    def __init__(self, fullTextFilter="", filters={}, fields="", offset=0, limit=0,
                 sortBy="id", sortOrder="ASCENDING", totalNumberOfRecords=0,
                 loadReferenceDepth=0):
        self.fullTextFilter = fullTextFilter
        self.filters = filters
        self.fields = fields
        self.offset = offset
        self.limit = limit
        self.sortBy = sortBy
        self.sortOrder = sortOrder
        self.totalNumberOfRecords = totalNumberOfRecords
        self.loadReferenceDepth = loadReferenceDepth



from pyopencell.responses.action_status import ActionStatus
from pyopencell.responses.paging import Paging


class PagedResponse():
    paging = {}
    action_status = {}

    def __init__(self, cls, **kwargs):
        self.paging = Paging(**kwargs.get("paging"))
        self.action_status = ActionStatus(**kwargs.get("actionStatus"))

        instances = []

        for instance in kwargs[cls._name]:
            instances.append(cls.items_resource_class(**instance))

        setattr(self, cls._name, instances)



class BaseResourceList:
    _name = ""
    _url_path = ""
    items_resource_class = None
    items_per_page = 10

    def __init__(self, kwargs_list):
        for base_resource in kwargs_list:
            for kwarg, value in base_resource.items():
                setattr(self, kwarg, value)



from pyopencell.client import Client
from pyopencell.resources.base_resource_list import BaseResourceList
from pyopencell.resources.invoice import Invoice
from pyopencell.responses.paged_response import PagedResponse
from pyopencell.responses.action_status import ActionStatus


class InvoiceList(BaseResourceList):
    _name = "invoices"
    _url_path = "/invoice/list"
    items_resource_class = Invoice
    items_per_page = 10

    invoices = []

    @classmethod
    def get(cls, query=None, limit=None, offset=None):
        """
        Returns a Invoice list with invoice instances data.

        :return: PagedResponse:
        """
        if not limit:
            limit = cls.items_per_page

        response_data = Client().get(
            cls._url_path,
            query=query,
            limit=limit,
            offset=offset
        )

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return PagedResponse(cls, **response_data)

    @classmethod
    def filter_by_date(cls, date, limit=None, offset=None):
        """
        Returns a Invoice list with invoice instances data.
        Filtering the invoices by date

        :arg date: Datetime
        :return: PagedResponse:
        """
        query = "invoiceDate:{}".format(date.strftime("%Y-%m-%d"))
        return cls.get(query=query, limit=limit, offset=offset)
